/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dgws;

import dk.sosi.seal.model.Request;
import dk.sundhedsdatastyrelsen.cbs.dgws.http.HttpClientService;
import dk.sundhedsdatastyrelsen.cbs.dgws.http.HttpClientServiceImpl;
import dk.sundhedsdatastyrelsen.cbs.dgws.http.HttpExecutor;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;

import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;

@CbsTest
@Disabled("Does actual call to STS")
@SuppressWarnings("java:S5979") /*SonarLint cannot see that @CbsTest has @ExtendWith(MockitoExtension.class)*/
class DGWSRequestFactoryIT {

    private HttpClientService httpClientService;

    private DGWSRequestFactory dgwsRequestFactory;

    @BeforeEach
    void setUp() {
        STSConfig stsConfig = new STSConfig(
            "http://test1.ekstern-test.nspop.dk:8080/sts/services/NewSecurityTokenService",
            "Sundhedsdatastyrelsen",
            "33257872",
            "Sundhedsdatastyrelsen - COVID-19 Borgerstatus.jks",
            "Test1234",
            "CBS"
        );

        httpClientService = Mockito.spy(new HttpClientServiceImpl(new HttpExecutor()));

        dgwsRequestFactory = new DGWSRequestFactory(stsConfig, httpClientService);
    }

    @Test
    void should_get_request_with_id_card() {
        dgwsRequestFactory.ensureValidIdCard();

        Request newRequest = dgwsRequestFactory.createNewRequest(UUID.randomUUID().toString());

        assertNotNull(newRequest.getIDCard());
    }

    @Test
    void should_not_refresh_if_id_card_is_not_expired() {

        //set new id card as current
        dgwsRequestFactory.ensureValidIdCard();

        // verify that first id card is requested from STS
        verify(httpClientService).soapRequest(any(), any(), any());

        // This call should not try to renew the id card
        dgwsRequestFactory.ensureValidIdCard();
        verifyNoMoreInteractions(httpClientService);
    }
}
