/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dgws.http;

import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.BasicHttpEntity;
import org.apache.http.message.BasicHttpResponse;
import org.apache.http.message.BasicStatusLine;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.io.ByteArrayInputStream;
import java.nio.charset.StandardCharsets;
import java.util.function.Function;

@CbsTest
@SuppressWarnings("java:S5979") /*SonarLint cannot see that @CbsTest has @ExtendWith(MockitoExtension.class)*/
class HttpClientServiceImplTest {

    private HttpClientServiceImpl httpClientService;
    private FakeHttpExecutor httpExecutor;

    @BeforeEach
    void setUp() {
        httpExecutor = new FakeHttpExecutor();
        httpClientService = new HttpClientServiceImpl(httpExecutor);
    }

    @Test
    void should_execute_request_and_extract_response() {
        String responseBody = "myResponse";
        BasicHttpEntity entity = new BasicHttpEntity();
        entity.setContent(new ByteArrayInputStream(responseBody.getBytes(StandardCharsets.UTF_8)));
        BasicHttpResponse response = new BasicHttpResponse(new BasicStatusLine(HttpVersion.HTTP_1_1, 200, "OK"));
        response.setEntity(entity);

        httpExecutor.setResponse(response);

        String soapResponse = httpClientService.soapRequest("http://my.endpoint.com", "myAction", "myRequest");

        Assertions.assertEquals(responseBody, soapResponse);
    }

    private class FakeHttpExecutor extends HttpExecutor {

        private HttpResponse response;

        @Override
        public <T> T post(HttpPost httpPost, Function<HttpResponse, T> responseHandler) {
            return responseHandler.apply(response);
        }

        public FakeHttpExecutor setResponse(HttpResponse response) {
            this.response = response;
            return this;
        }
    }
}
