/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dgws.http;

import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import org.apache.http.HttpVersion;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.BasicHttpEntity;
import org.apache.http.message.BasicHttpResponse;
import org.apache.http.message.BasicStatusLine;
import org.apache.http.util.EntityUtils;
import org.junit.jupiter.api.Test;

import java.io.ByteArrayInputStream;
import java.net.URI;
import java.nio.charset.StandardCharsets;

import static org.junit.jupiter.api.Assertions.assertAll;
import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

@CbsTest
@SuppressWarnings("java:S5979") /*SonarLint cannot see that @CbsTest has @ExtendWith(MockitoExtension.class)*/
class HttpUtilTest {

    @Test
    void should_create_request_with_action_body_and_endpoint() {
        String endpoint = "http://my.endpoint.com";
        String soapAction = "myAction";
        String requestXml = "myRequest";

        HttpPost soapRequest = HttpUtil.createSoapRequest(endpoint, soapAction, requestXml);

        assertAll(
            () -> assertEquals(URI.create(endpoint), soapRequest.getURI()),
            () -> assertEquals(soapAction, soapRequest.getFirstHeader("SOAPAction").getValue()),
            () -> assertEquals(requestXml, EntityUtils.toString(soapRequest.getEntity()))
        );
    }

    @Test
    void should_throw_if_status_is_not_ok() {
        BasicHttpResponse response = new BasicHttpResponse(new BasicStatusLine(HttpVersion.HTTP_1_1, 500, "Internal server error"));
        assertThrows(HttpException.class, () -> HttpUtil.checkSoapResponseStatus("endpoint", "action", response));
    }

    @Test
    void should_not_throw_if_status_is_ok() {
        BasicHttpResponse response = new BasicHttpResponse(new BasicStatusLine(HttpVersion.HTTP_1_1, 200, "OK"));
        assertDoesNotThrow(() -> HttpUtil.checkSoapResponseStatus("endpoint", "action", response));
    }

    @Test
    void should_extract_response_body() {
        String responseBody = "myResponse";

        BasicHttpEntity entity = new BasicHttpEntity();
        entity.setContent(new ByteArrayInputStream(responseBody.getBytes(StandardCharsets.UTF_8)));

        BasicHttpResponse response = new BasicHttpResponse(new BasicStatusLine(HttpVersion.HTTP_1_1, 200, "OK"));
        response.setEntity(entity);

        String extractResponse = HttpUtil.extractResponse(response);

        assertEquals(extractResponse, responseBody);
    }

}
