/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.kafka;

import dk.sundhedsdatastyrelsen.cbs.dto.impl.kafka.DataFetcherJobBuilder;
import dk.sundhedsdatastyrelsen.cbs.dto.kafka.DataFetcherJob;
import dk.sundhedsdatastyrelsen.cbs.dto.kafka.JobType;
import org.apache.kafka.clients.producer.Producer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.log4j.Logger;

import javax.annotation.PreDestroy;
import java.util.Properties;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;

public class CbsKafkaProducer {
    private static final Logger log = Logger.getLogger(CbsKafkaProducer.class);
    private final AtomicBoolean closed = new AtomicBoolean(false);
    private final String clientId;
    private final String topic;
    private final Producer<String, DataFetcherJob> producer;

    public CbsKafkaProducer(String topic, Properties kafkaProperties, Producer<String, DataFetcherJob> producer) {
        this.clientId = kafkaProperties.getProperty(ProducerConfig.CLIENT_ID_CONFIG);
        this.topic = topic;
        this.producer = producer;
    }

    public void sendDataFetcherJob(String cpr, JobType jobType, String jobId, String idwsMessageId) {
        final DataFetcherJob job = DataFetcherJobBuilder.aDataFetcherJob()
            .withCpr(cpr)
            .withMsgId(idwsMessageId)
            .withType(jobType)
            .withJobId(jobId)
            .build();
        if (log.isTraceEnabled()) {
            log.trace(clientId + " sending: \n" +
                "\t Key: <redacted>\n" +
                "\t Value:" + job + "\n" +
                "\t Topic:" + topic);
        }
        producer.send(new ProducerRecord<>(topic, cpr, job)); //key decides which partition the records ends on and we want each citizen assigned a specific partition
    }


    @PreDestroy
    public void close() {
        if (!closed.getAndSet(true)) {
            log.info("Closing " + clientId);
            producer.close(1, TimeUnit.MINUTES);
        }
    }
}
