/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.kafka;

import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import dk.sundhedsdatastyrelsen.cbs.test.TestConfiguration;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.ConfigurableEnvironment;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;

import java.util.Properties;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;

@CbsTest
@SpringJUnitConfig(TestConfiguration.class)
class CbsKafkaUtilsTest {
    @Autowired
    private ConfigurableEnvironment env;

    @Test
    void test_kafka_properties() {
        final Properties kafkaProperties = CbsKafkaUtils.getKafkaProperties("my.kafka.prefix.", env);

        assertFalse(kafkaProperties.containsKey("kafka.topic"));
        assertEquals("somehost:9999", kafkaProperties.get("bootstrap.servers"));
        assertEquals("unittest", kafkaProperties.get("client.id"));
        assertEquals("unit-test-component", kafkaProperties.get("nsp.kafka.producer.component.name"));
    }

    @Test
    void test_app_friendly_unique_producer_client_id() {
        final Properties kafkaProperties = new Properties();
        String clientId = "my-app-producer";
        kafkaProperties.put(ProducerConfig.CLIENT_ID_CONFIG, clientId);

        final Properties instanceProperties = CbsKafkaUtils.getProducerInstanceProperties(kafkaProperties);
        assertEquals(clientId + "-1", instanceProperties.getProperty(ProducerConfig.CLIENT_ID_CONFIG));

        final Properties instanceProperties2 = CbsKafkaUtils.getProducerInstanceProperties(kafkaProperties);
        assertEquals(clientId + "-2", instanceProperties2.getProperty(ProducerConfig.CLIENT_ID_CONFIG));
    }


    @Test
    void test_app_friendly_unique_consumer_client_id() {
        final Properties kafkaProperties = new Properties();
        String clientId = "my-app-consumer";
        kafkaProperties.put(ConsumerConfig.CLIENT_ID_CONFIG, clientId);

        final Properties instanceProperties = CbsKafkaUtils.getConsumerInstanceProperties(kafkaProperties);
        assertEquals(clientId + "-1", instanceProperties.getProperty(ProducerConfig.CLIENT_ID_CONFIG));

        final Properties instanceProperties2 = CbsKafkaUtils.getConsumerInstanceProperties(kafkaProperties);
        assertEquals(clientId + "-2", instanceProperties2.getProperty(ProducerConfig.CLIENT_ID_CONFIG));
    }


}
