/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.nas.facade;

import dk.medcom.dgws._2006._04.dgws_1_0.Header;
import dk.sosi.seal.model.Request;
import dk.sundhedsdatastyrelsen.cbs.dgws.DGWSRequestFactory;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import java.util.UUID;

public class DefaultDGWSHeaderProvider implements DGWSHeaderProvider {

    private final DGWSRequestFactory dgwsRequestFactory;
    private final Unmarshaller unmarshaller;

    public DefaultDGWSHeaderProvider(DGWSRequestFactory dgwsRequestFactory) throws JAXBException {
        this.dgwsRequestFactory = dgwsRequestFactory;
        this.unmarshaller = JAXBContext.newInstance(Security.class, Header.class).createUnmarshaller();
    }

    @Override
    public DGWSHeaders getHeaders() {
        dgwsRequestFactory.ensureValidIdCard();
        Request request = dgwsRequestFactory.createNewRequest(UUID.randomUUID().toString());
        Document document = request.serialize2DOMDocument();
        document.getDocumentElement().getFirstChild().getChildNodes();
        NodeList headers = document.getDocumentElement().getFirstChild().getChildNodes();

        Security security = null;
        Header header = null;

        for (int i = 0; i < headers.getLength(); i++) {
            Node element = headers.item(i);
            if (element.getLocalName().equalsIgnoreCase("Security")) {
                security = unmarshal(element, Security.class);
            } else if (element.getLocalName().equalsIgnoreCase("Header")) {
                header = unmarshal(element, Header.class);
            }
        }

        return new DGWSHeaders(security, header);
    }

    private <T> T unmarshal(Node element, Class<T> declaredType) {
        try {
            return unmarshaller.unmarshal(element, declaredType).getValue();
        } catch (JAXBException e) {
            return null;
        }
    }


}
