/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.nas.facade;

import dk.nsi.advis.v10.NotifyContent;
import org.oasis_open.docs.wsn.b_2.NotificationMessageHolderType;

import javax.xml.bind.JAXBElement;

public class NasPayloadExtractor {

    public static <T> T getMessageContent(NotificationMessageHolderType messageHolderType, Class<T> declaredType) {
        Object content = messageHolderType.getMessage().getAny();

        if (declaredType.isInstance(content)) {
            return declaredType.cast(content);
        }

        return null;
    }

    public static Object getMessageContentPayload(NotificationMessageHolderType messageHolderType) {
        NotifyContent messageContent = getMessageContent(messageHolderType, NotifyContent.class);

        if (messageContent == null) {
            return null;
        }

        Object payload = messageContent.getAny();

        if (payload instanceof JAXBElement) {
            JAXBElement<?> jaxbElement = (JAXBElement<?>) payload;
            return jaxbElement.getValue();
        }

        return payload;
    }

    public static <T> T getMessageContentPayload(NotificationMessageHolderType messageHolderType, Class<T> declaredType) {
        Object messageContentPayload = getMessageContentPayload(messageHolderType);

        if (declaredType.isInstance(messageContentPayload)) {
            return declaredType.cast(messageContentPayload);
        }

        return null;
    }

}
