/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.nas.facade;

import dk.nsi.advis.v10.NotifyContent;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.JobStatusNotification;
import org.oasis_open.docs.wsn.b_2.NotificationMessageHolderType;
import org.oasis_open.docs.wsn.b_2.Notify;
import org.oasis_open.docs.wsn.b_2.TopicExpressionType;
import org.oasis_open.docs.wsn.bw_2.InvalidTopicExpressionFault;
import org.oasis_open.docs.wsn.bw_2.NotificationConsumer;
import org.oasis_open.docs.wsn.bw_2.NotifyFailedFault;

import javax.xml.bind.JAXBElement;

public class NasPublisherImpl implements NasPublisher {

    private final org.oasis_open.docs.wsn.b_2.ObjectFactory b2ObjectFactory = new org.oasis_open.docs.wsn.b_2.ObjectFactory();
    private final dk.nsi.advis.v10.ObjectFactory nsiObjectFactory = new dk.nsi.advis.v10.ObjectFactory();
    private final dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.ObjectFactory cbsObjectFactory = new dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.ObjectFactory();

    private final NotificationConsumer notificationConsumer;
    private final String topic;

    public NasPublisherImpl(NotificationConsumer notificationConsumer, String topic) {
        this.notificationConsumer = notificationConsumer;
        this.topic = topic;
    }

    @Override
    public void publish(JobStatusNotification jobStatusNotification, String cpr) {
        JAXBElement<JobStatusNotification> payload = cbsObjectFactory.createJobStatusNotification(jobStatusNotification);

        NotifyContent notifyContent = nsiObjectFactory.createNotifyContent();
        notifyContent.setAny(payload);
        notifyContent.setId(cpr);
        notifyContent.setIdType(Constants.ID_TYPE_CPR);
        notifyContent.setIsSystemNotification(false);

        NotificationMessageHolderType.Message message = b2ObjectFactory.createNotificationMessageHolderTypeMessage();
        message.setAny(notifyContent);

        TopicExpressionType topicExpressionType = b2ObjectFactory.createTopicExpressionType();
        topicExpressionType.setDialect(Constants.TOPIC_EXPRESSION_SIMPLE);
        topicExpressionType.getContent().add(topic);

        NotificationMessageHolderType notificationMessageHolderType = b2ObjectFactory.createNotificationMessageHolderType();
        notificationMessageHolderType.setMessage(message);
        notificationMessageHolderType.setTopic(topicExpressionType);

        Notify notify = b2ObjectFactory.createNotify();
        notify.getNotificationMessage().add(notificationMessageHolderType);

        try {
            notificationConsumer.notify(notify);
        } catch (InvalidTopicExpressionFault | NotifyFailedFault e) {
            throw new NasException(String.format("Failed to publish notification with job id '%s'", jobStatusNotification.getJobId()), e);
        }
    }
}
