/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.nas.facade;

import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.JobStatus;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.JobStatusNotification;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.ObjectFactory;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.oasis_open.docs.wsn.b_2.Notify;
import org.oasis_open.docs.wsn.bw_2.NotificationConsumer;
import org.oasis_open.docs.wsn.bw_2.NotifyFailedFault;

import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.any;


@CbsTest
@SuppressWarnings("java:S5979") /*SonarLint cannot see that @CbsTest has @ExtendWith(MockitoExtension.class)*/
class NasPublisherImplTest {

    public static final String TOPIC = "topic";

    @Mock
    private NotificationConsumer notificationConsumer;

    @Captor
    private ArgumentCaptor<Notify> notifyArgumentCaptor;

    private NasPublisherImpl nasPublisher;

    @BeforeEach
    void setUp() {
        nasPublisher = new NasPublisherImpl(notificationConsumer, TOPIC);
    }

    @Test
    void should_publish_notification_to_topic() throws Exception {
        ObjectFactory objectFactory = new ObjectFactory();
        JobStatusNotification jobStatusNotification = objectFactory.createJobStatusNotification();
        jobStatusNotification.setJobId(UUID.randomUUID().toString());
        jobStatusNotification.setJobStatus(JobStatus.OK);

        nasPublisher.publish(jobStatusNotification, "0506504003");

        Mockito.verify(notificationConsumer).notify(notifyArgumentCaptor.capture());

        Notify notify = notifyArgumentCaptor.getValue();

        assertEquals(TOPIC, notify.getNotificationMessage().get(0).getTopic().getContent().get(0).toString());
    }

    @Test
    void should_fail_if_unable_to_notify() throws Exception {
        Mockito.when(notificationConsumer.notify(any())).thenThrow(new NotifyFailedFault());

        ObjectFactory objectFactory = new ObjectFactory();
        JobStatusNotification jobStatusNotification = objectFactory.createJobStatusNotification();
        jobStatusNotification.setJobId(UUID.randomUUID().toString());
        jobStatusNotification.setJobStatus(JobStatus.OK);

        assertThrows(NasException.class, () -> nasPublisher.publish(jobStatusNotification, "0506504003"));

    }


}
