/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.test;


import org.awaitility.Durations;
import org.awaitility.core.ThrowingRunnable;

import static org.awaitility.Awaitility.await;

public class AssertionTestUtil {

    public static void waitForAsserted(ThrowingRunnable assertion) {
        await()
                .atMost(Durations.FIVE_SECONDS)
                .with().pollInterval(Durations.ONE_HUNDRED_MILLISECONDS)
                .untilAsserted(assertion);
    }

    public static void ensureAssertedFor500Millis(ThrowingRunnable assertion) {
        await()
                .during(Durations.TWO_HUNDRED_MILLISECONDS) //assertion must hold at least 200ms
                .atMost(Durations.FIVE_HUNDRED_MILLISECONDS) //stop checking after 500ms
                .with().pollDelay(Durations.TWO_HUNDRED_MILLISECONDS) //wait with checking until 200ms has passed
                .with().pollInterval(Durations.ONE_HUNDRED_MILLISECONDS)
                .untilAsserted(assertion);
    }
}
