/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.impl;

import dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.CacheInvalidationRecord;
import dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.RecordInvalidator;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.NasNotificationProcessor;
import org.oasis_open.docs.wsn.b_2.NotificationMessageHolderType;

import java.util.List;

public class CacheInvalidatingNasNotificationProcessor implements NasNotificationProcessor {

    private final RecordInvalidator recordInvalidator;
    private final RecordMapper recordMapper;

    public CacheInvalidatingNasNotificationProcessor(RecordInvalidator recordInvalidator, RecordMapper recordMapper) {
        this.recordInvalidator = recordInvalidator;
        this.recordMapper = recordMapper;
    }

    @Override
    public void process(List<NotificationMessageHolderType> messages) {
        for (NotificationMessageHolderType message : messages) {
            CacheInvalidationRecord record = recordMapper.toRecord(message);
            if (record != null) {
                invalidateRecord(record);
            }
        }
    }

    private void invalidateRecord(CacheInvalidationRecord record) {
        boolean invalidated = recordInvalidator.invalidate(record);

        if (!invalidated) {
            handleRecordNotInvalidated(record);
        }
    }

    private void handleRecordNotInvalidated(CacheInvalidationRecord record) {
        throw new DataCacheInvalidationException(String.format("Could not invalidate record. Trace = %s", record.getTraceId()));
    }


}
