/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.impl;


import dk.sundhedsdatastyrelsen.cbs.dao.VaccinationRecord;
import dk.sundhedsdatastyrelsen.cbs.dao.impl.jdbc.VaccinationRecordImpl;
import dk.sundhedsdatastyrelsen.cbs.datacache.Writer;
import dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.CacheInvalidationRecord;
import dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.RecordInvalidator;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.Vaccination;
import dk.vaccinationsregister.schemas._2013._12._01.GetVaccinationPassportAsPDFResponseType;

public class VaccinationRecordInvalidator implements RecordInvalidator {

    private final Writer<VaccinationRecord<Vaccination, GetVaccinationPassportAsPDFResponseType>> writer;

    public VaccinationRecordInvalidator(Writer<VaccinationRecord<Vaccination, GetVaccinationPassportAsPDFResponseType>> writer) {
        this.writer = writer;
    }

    @Override
    public boolean invalidate(CacheInvalidationRecord record) {
        VaccinationRecordImpl<Vaccination, GetVaccinationPassportAsPDFResponseType> vaccinationRecord = new VaccinationRecordImpl<Vaccination, GetVaccinationPassportAsPDFResponseType>()
            .cpr(record.getCpr())
            .traceId(record.getTraceId())
            .invalidationTime(record.getInvalidationTime());

        return writer.write(vaccinationRecord);
    }
}
