/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datacache.invalidation;

import dk.sundhedsdatastyrelsen.cbs.dao.LabResultRecord;
import dk.sundhedsdatastyrelsen.cbs.dao.VaccinationRecord;
import dk.sundhedsdatastyrelsen.cbs.datacache.Writer;
import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResult;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.Vaccination;
import dk.sundhedsdatastyrelsen.kids.dao.KidsLabResult;
import dk.vaccinationsregister.schemas._2013._12._01.GetVaccinationPassportAsPDFResponseType;
import org.apache.log4j.Logger;
import org.mockito.Answers;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.context.annotation.PropertySource;
import org.springframework.lang.NonNull;

import javax.sql.DataSource;

import java.util.Collection;

import static org.mockito.Mockito.mock;

@Configuration
@PropertySource("classpath:application-datacache-invalidator.properties")
public class TestConfig {

    Logger log = Logger.getLogger("TestWriter");

    @Bean
    @NonNull
    public Writer<VaccinationRecord<Vaccination, GetVaccinationPassportAsPDFResponseType>> vaccinationWriter() {
        return record -> {
            log.info("Writing VaccinationRecord " + record.traceId() + " :: " + record.invalidationTime());
            return true;
        };
    }

    @Bean
    @NonNull
    public Writer<LabResultRecord<LabResult, Collection<KidsLabResult>>> labResultWriter() {
        return record -> {
            log.info("Writing LabResultRecord " + record.traceId() + " :: " + record.invalidationTime());
            return true;
        };
    }

    @Bean
    @Primary
    public DataSource dataSource() {
        return mock(DataSource.class, Answers.RETURNS_DEEP_STUBS);
    }
}
