/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.impl;

import dk.sundhedsdatastyrelsen.cbs.nas.facade.EndpointReferences;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.NasClient;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;

import javax.xml.ws.wsaddressing.W3CEndpointReference;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotEquals;
import static org.mockito.Mockito.atMostOnce;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@CbsTest
@SuppressWarnings("java:S5979") /*SonarLint cannot see that @CbsTest has @ExtendWith(MockitoExtension.class)*/
class InMemoryNasSubscriptionRepositoryTest {

    @Mock
    private NasClient nasClient;
    private InMemoryNasSubscriptionRepository repository;

    @BeforeEach
    void setUp() {
        repository = new InMemoryNasSubscriptionRepository(nasClient);
    }

    @Test
    void should_create_new_pullpoint_on_first_call() {
        String address = "http:///pullpoint.com";
        when(nasClient.createPullPoint()).thenReturn(EndpointReferences.newEndpointReference(address));

        W3CEndpointReference pullPoint = repository.getPullPoint("myTopic");

        verify(nasClient).createPullPoint();

        assertEquals(address, EndpointReferences.extractAddress(pullPoint));
    }

    @Test
    void should_return_saved_pullpoint_on_second_call() {
        String address = "http:///pullpoint.com";
        when(nasClient.createPullPoint()).thenReturn(EndpointReferences.newEndpointReference(address));

        W3CEndpointReference pullPoint = repository.getPullPoint("myTopic");

        assertEquals(address, EndpointReferences.extractAddress(pullPoint));

        pullPoint = repository.getPullPoint("myTopic");

        assertEquals(address, EndpointReferences.extractAddress(pullPoint));

        verify(nasClient, atMostOnce()).createPullPoint();
    }

    @Test
    void should_return_same_message_id_until_commit() {

        String topic = "myTopic";
        UUID messageId = repository.getNextMessageId(topic);

        assertEquals(messageId, repository.getNextMessageId(topic));
        assertEquals(messageId, repository.getNextMessageId(topic));
        assertEquals(messageId, repository.getNextMessageId(topic));
        assertEquals(messageId, repository.getNextMessageId(topic));
        assertEquals(messageId, repository.getNextMessageId(topic));

        repository.commit(topic);

        assertNotEquals(messageId, repository.getNextMessageId(topic));
    }
}
