/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dao.impl.jdbc;

import dk.sundhedsdatastyrelsen.cbs.dao.Record;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.time.LocalDateTime;

abstract class RecordImpl<T, U, V extends RecordImpl<T, U, V>> implements Record<T, U> {

    @NonNull static final String CPR_COLUMN_LABEL = "CPR";
    @NonNull static final String CREATION_TIME_COLUMN_LABEL = "CREATION_TIME";
    @NonNull static final String TRACE_ID_COLUMN_LABEL = "TRACE_ID";
    @NonNull static final String INVALIDATION_TIME_COLUMN_LABEL = "INVALIDATION_TIME";
    @NonNull static final String DATA_COLUMN_LABEL = "DATA";
    @NonNull static final String SOURCE_COLUMN_LABEL = "SOURCE";

    @NonNull private final V actual;
    private String cpr;
    private LocalDateTime creationTime;
    private String traceId;
    private LocalDateTime invalidationTime;
    private T data;
    private U source;

    @SuppressWarnings("unchecked")
    RecordImpl() {
        actual = (V) this;
    }

    @Override
    public String cpr() {
        return cpr;
    }

    @NonNull
    public V cpr(@NonNull final String cpr) {
        this.cpr = cpr;
        return actual;
    }

    @Override
    public String traceId() {
        return traceId;
    }

    @NonNull
    public V traceId(@NonNull final String traceId) {
        this.traceId = traceId;
        return actual;
    }

    @Override
    public LocalDateTime creationTime() {
        return creationTime;
    }

    @NonNull
    public V creationTime(@NonNull final LocalDateTime creationTime) {
        this.creationTime = creationTime;
        return actual;
    }

    @Override
    public LocalDateTime invalidationTime() {
        return invalidationTime;
    }

    @NonNull
    public V invalidationTime(@Nullable final LocalDateTime invalidationTime) {
        this.invalidationTime = invalidationTime;
        return actual;
    }

    @Override
    public T data() {
        return data;
    }

    @NonNull
    public V setData(@Nullable final T data) {
        this.data = data;
        return actual;
    }

    @Override
    public U source() {
        return source;
    }

    @NonNull
    public V setSource(@Nullable final U source) {
        this.source = source;
        return actual;
    }
}
