/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dao;

import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.time.LocalDateTime;

public interface Record<T, U> {
    String CATALOG_NAME = "cbs-datacache";
    String SCHEMA_NAME = null;

    /**
     * Gets the cpr number for this record
     * @return the universal unique key
     */
    @NonNull
    String cpr();

    /**
     * Gets the trace identifier for the creation of this record
     * @return the trace identifier
     */
    @NonNull
    String traceId();

    /**
     * Gets the time when this record was created
     * @return the time
     */
    @NonNull
    LocalDateTime creationTime();

    /**
     * Gets the invalidation time for this record.
     * This value is used as an synchronization point between concurrent invalidation and write events.
     * @return The time of the invalidation, if <code>null</code> means the record is not invalidated.
     */
    @Nullable
    LocalDateTime invalidationTime();

    /**
     * Gets the data (dto) of this record
     * @return the data (dto), may be <code>null</code>
     */
    @Nullable
    T data();

    /**
     * Gets the raw data (dto) of this record
     * @return the raw data (dto), may be <code>null</code>
     */
    @Nullable
    U source();
}
