/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datafetcher.config;

import dk.sundhedsdatastyrelsen.cbs.datafetcher.ddv.DDVVaccinePasService;
import dk.sundhedsdatastyrelsen.cbs.datafetcher.ddv.DGWSHelper;
import dk.sundhedsdatastyrelsen.cbs.datafetcher.ddv.WhitelistingConfig;
import dk.sundhedsdatastyrelsen.cbs.dgws.DGWSRequestFactory;
import dk.sundhedsdatastyrelsen.cbs.dgws.STSConfig;
import dk.sundhedsdatastyrelsen.cbs.dgws.http.HttpClientService;
import dk.sundhedsdatastyrelsen.cbs.dgws.http.HttpClientServiceImpl;
import dk.sundhedsdatastyrelsen.cbs.dgws.http.HttpExecutor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;

@Configuration
@PropertySource("classpath:application-datafetcher.properties")
@SuppressWarnings("java:S1118")
//SonarLint thinks this is a utility class due to the static modifier (which is intended for PropertySourcesPlaceholderConfigurer as per @Bean JavaDoc)
public class DDVConfig {

    @Bean
    public DGWSRequestFactory dgwsRequestFactory(STSConfig stsConfig, HttpClientService httpClientService) {
        return new DGWSRequestFactory(stsConfig, httpClientService);
    }

    @Bean
    public DGWSHelper dgwsHelper(WhitelistingConfig whitelistingConfig, DGWSRequestFactory dgwsRequestFactory) {
        return new DGWSHelper(whitelistingConfig, dgwsRequestFactory);
    }

    @Bean
    public HttpClientService httpClientService() {
        return new HttpClientServiceImpl(new HttpExecutor());
    }

    @Bean
    public DDVVaccinePasService ddvVaccinePasService(DGWSHelper dgwsHelper, @Value("${ddv.service.endpoint}") String endpoint, HttpClientService httpClientService) {
        return new DDVVaccinePasService(dgwsHelper, endpoint, httpClientService);
    }

    @Bean
    public WhitelistingConfig whitelistingConfig(
        @Value("${ddv.whitelisting.systemname}") String systemName,
        @Value("${ddv.whitelisting.systemownername}") String systemOwnerName,
        @Value("${ddv.whitelisting.systemversion}") String systemVersion,
        @Value("${ddv.whitelisting.orgusingidcvr}") String orgUsingIdCVR,
        @Value("${ddv.whitelisting.orgusingidname}") String orgUsingIdName,
        @Value("${ddv.whitelisting.requestedrole}") String requestedRole) {
        return new WhitelistingConfig(systemName, systemOwnerName, systemVersion, orgUsingIdCVR, orgUsingIdName, requestedRole);
    }

    @Bean
    public STSConfig stsConfig(
        @Value("${ddv.idcard.sts.endpoint}") String stsEndpoint,
        @Value("${ddv.idcard.name}") String organizationName,
        @Value("${ddv.idcard.cvr}") String organizationCode,
        @Value("${ddv.idcard.keystore.path}") String keystorePath,
        @Value("${ddv.idcard.keystore.password}") String keystorePassword,
        @Value("${ddv.idcard.systemName}") String systemName) {
        return new STSConfig(stsEndpoint, organizationName, organizationCode, keystorePath, keystorePassword, systemName);
    }

}
