/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datafetcher.config;


import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

@Component
public class DataFetcherConfigProperties {
    private int maxConcurrentProcessings;
    private int processingBeginTimeoutMillis;
    private int processingEndTimeoutMillis;
    private int shutdownTimeoutMillis;

    public DataFetcherConfigProperties(@Value("${datafetcher.max-concurrent-processings:5}") int maxConcurrentProcessings,
                                       @Value("${datafetcher.processing-begin-timeout-millis:10000}") int processingBeginTimeoutMillis,
                                       @Value("${datafetcher.processing-end-timeout-millis:300000}") int processingEndTimeoutMillis,
                                       @Value("${datafetcher.shutdown-timeout-millis:30000}") int shutdownTimeoutMillis) {
        this.maxConcurrentProcessings = maxConcurrentProcessings;
        this.processingBeginTimeoutMillis = processingBeginTimeoutMillis;
        this.processingEndTimeoutMillis = processingEndTimeoutMillis;
        this.shutdownTimeoutMillis = shutdownTimeoutMillis;
    }

    public int getMaxConcurrentProcessings() {
        return maxConcurrentProcessings;
    }

    public DataFetcherConfigProperties setMaxConcurrentProcessings(int maxConcurrentProcessings) {
        this.maxConcurrentProcessings = maxConcurrentProcessings;
        return this;
    }

    public int getProcessingBeginTimeoutMillis() {
        return processingBeginTimeoutMillis;
    }

    public DataFetcherConfigProperties setProcessingBeginTimeoutMillis(int processingBeginTimeoutMillis) {
        this.processingBeginTimeoutMillis = processingBeginTimeoutMillis;
        return this;
    }

    public int getProcessingEndTimeoutMillis() {
        return processingEndTimeoutMillis;
    }

    public DataFetcherConfigProperties setProcessingEndTimeoutMillis(int processingEndTimeoutMillis) {
        this.processingEndTimeoutMillis = processingEndTimeoutMillis;
        return this;
    }

    public int getShutdownTimeoutMillis() {
        return shutdownTimeoutMillis;
    }

    public DataFetcherConfigProperties setShutdownTimeoutMillis(int shutdownTimeoutMillis) {
        this.shutdownTimeoutMillis = shutdownTimeoutMillis;
        return this;
    }
}
