/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datafetcher.ddv;

import dk.sdsd.dgws._2012._06.ObjectFactory;
import dk.sdsd.dgws._2012._06.WhiteListingHeader;
import dk.sosi.seal.model.Request;
import dk.sosi.seal.xml.XmlUtil;
import dk.sundhedsdatastyrelsen.cbs.dgws.DGWSRequestFactory;
import org.w3c.dom.Document;
import org.xmlsoap.schemas.soap.envelope.Body;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.transform.dom.DOMResult;
import java.util.UUID;

public class DGWSHelper {

    private final DGWSRequestFactory dgwsRequestFactory;

    private final String flowId = UUID.randomUUID().toString();

    private final WhitelistingConfig whitelistingConfig;

    public DGWSHelper(WhitelistingConfig whitelistingConfig, DGWSRequestFactory dgwsRequestFactory) {
        this.whitelistingConfig = whitelistingConfig;
        this.dgwsRequestFactory = dgwsRequestFactory;
    }

    public void ensureValidIdCard() {
        dgwsRequestFactory.ensureValidIdCard();
    }

    public String addHeaders(JAXBContext jaxbContext, Body soapBody) throws JAXBException {
        Request sosiRequest = dgwsRequestFactory.createNewRequest(flowId); // Flow id provided and Message ID generated

        WhiteListingHeader h = whitelistingConfig.createWhiteListingHeader();
        sosiRequest.addNonSOSIHeader(jaxb2Element(jaxbContext, new ObjectFactory().createWhiteListingHeader(h)));
        sosiRequest.setBody(jaxb2Element(jaxbContext, soapBody.getAny().get(0)));

        Document document = sosiRequest.serialize2DOMDocument();
        return XmlUtil.node2String(document);
    }

    private org.w3c.dom.Element jaxb2Element(JAXBContext jaxbContext, Object jaxbObject) throws JAXBException {
        DOMResult res = new DOMResult();
        jaxbContext.createMarshaller().marshal(jaxbObject, res);
        return ((Document) res.getNode()).getDocumentElement();
    }
}
