/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datafetcher.servlet;

import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Answers;
import org.mockito.Mock;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;

import static dk.sundhedsdatastyrelsen.cbs.datafetcher.servlet.DataFetcherServletContextListener.SPRING_APPLICATION_CONTEXT;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@CbsTest
@SuppressWarnings("java:S5979") /*SonarLint cannot see that @CbsTest has @ExtendWith(MockitoExtension.class)*/
class DataFetcherServletTest {
    private final ApplicationContext testApplicationContext = new AnnotationConfigApplicationContext("dk.sundhedsdatastyrelsen.cbs.test");
    @Mock
    PrintWriter responseBodyWriter;
    @Mock(answer = Answers.RETURNS_DEEP_STUBS)
    private HttpServletRequest httpServletRequest;
    @Mock
    private HttpServletResponse httpServletResponse;
    private final DataFetcherServlet dataFetcherServlet = new DataFetcherServlet();

    @BeforeEach
    void setUp() throws IOException {
        when(httpServletRequest.getServletContext().getAttribute(SPRING_APPLICATION_CONTEXT)).thenReturn(testApplicationContext);
        when(httpServletResponse.getWriter()).thenReturn(responseBodyWriter);
    }

    @Test
    void test_fetch_0_jobs() {
        dataFetcherServlet.doGet(httpServletRequest, httpServletResponse);
        verify(httpServletResponse).setStatus(200);
        verify(httpServletResponse).setContentType("text/plain; charset=utf-8");
        verify(responseBodyWriter).write("Processed 0 jobs\n" +
            "SUCCESS: 0\n" +
            "FAILED : 0\n");
    }
}
