/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.kafka.config;

import dk.sds.nsp.kafka.clients.consumer.NSPKafkaConsumer;
import dk.sundhedsdatastyrelsen.cbs.datafetcher.config.DataFetcherKafkaConfig;
import dk.sundhedsdatastyrelsen.cbs.dto.kafka.DataFetcherJob;
import dk.sundhedsdatastyrelsen.cbs.kafka.CbsKafkaConsumer;
import dk.sundhedsdatastyrelsen.cbs.kafka.pooling.CbsKafkaConsumerPool;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import dk.sundhedsdatastyrelsen.cbs.test.TestConfiguration;
import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.ConfigurableEnvironment;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;

import java.util.Collections;
import java.util.Set;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@CbsTest
@SpringJUnitConfig({TestConfiguration.class, CbsKafkaConfig.class})
@SuppressWarnings("java:S5979") /*SonarLint cannot see that @CbsTest has @ExtendWith(MockitoExtension.class)*/
class DataFetcherKafkaConfigTest {
    @Autowired
    private ConfigurableEnvironment env;
    @Autowired
    private GenericObjectPoolConfig<CbsKafkaConsumer> poolConfig;

    @Mock
    private NSPKafkaConsumer<String, DataFetcherJob> nspKafkaConsumer;
    private DataFetcherKafkaConfig dataFetcherKafkaConfig;

    @BeforeEach
    void setUp() {
        dataFetcherKafkaConfig = new DataFetcherKafkaConfig(env);
    }

    @Test
    void test_cbsKafkaConsumerPool_bean_method() throws Exception {
        int pollTimeout = 1;
        when(nspKafkaConsumer.poll(pollTimeout)).thenReturn(new ConsumerRecords<>(Collections.emptyMap()));

        CbsKafkaConsumerPool cbsKafkaConsumerPool = dataFetcherKafkaConfig.cbsKafkaConsumerPool(pollTimeout, poolConfig, (p) -> nspKafkaConsumer);
        Set<DataFetcherJob> dataFetcherJobs = cbsKafkaConsumerPool.pollDataFetcherJobs();
        assertEquals(0, dataFetcherJobs.size());

        verify(nspKafkaConsumer).poll(pollTimeout);
    }

    @Test
    void test_missing_kafka_property_gives_exception() {
        System.setProperty("kafka.topic", "");
        IllegalArgumentException illegalArgumentException = assertThrows(IllegalArgumentException.class, () -> dataFetcherKafkaConfig.cbsKafkaConsumerPool(1, poolConfig, (p) -> nspKafkaConsumer));
        assertEquals(DataFetcherKafkaConfig.MSG_KAFKA_TOPIC_PROPERTY_MISSING, illegalArgumentException.getMessage());
        System.clearProperty("kafka.topic");
        assertDoesNotThrow(() -> dataFetcherKafkaConfig.cbsKafkaConsumerPool(1, poolConfig, (p) -> nspKafkaConsumer));
    }
}
