/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.test;

import dk.sundhedsdatastyrelsen.cbs.datafetcher.DataFetcher;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.context.annotation.PropertySource;

import javax.sql.DataSource;
import java.sql.*;
import java.util.concurrent.atomic.AtomicBoolean;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

@Configuration
@PropertySource("classpath:application-datafetcher.properties")
public class TestConfiguration {
    private final DataSource dataSource;

    @Primary
    @Bean
    public DataFetcher dataFetcher() {
        return mock(DataFetcher.class);
    }

    @Bean
    public DataSource dataSource() {
        return dataSource;
    }

    TestConfiguration() {
        dataSource = mock(DataSource.class);
        final Connection connection = mock(Connection.class);
        final DatabaseMetaData metaData = mock(DatabaseMetaData.class);
        final ResultSet tableResultSet = mock(ResultSet.class);
        final ResultSet columnResultSet = mock(ResultSet.class);
        final PreparedStatement preparedStatement = mock(PreparedStatement.class);

        final AtomicBoolean nextTable = new AtomicBoolean(true);
        final AtomicBoolean nextColumn = new AtomicBoolean(true);

        try {
            when(dataSource.getConnection()).thenReturn(connection);
            when(connection.getMetaData()).thenReturn(metaData);
            when(metaData.getTables(any(), any(), any(), any())).then(i -> {
                when(tableResultSet.getString("TABLE_SCHEM")).then(j -> i.getArgument(1));
                when(tableResultSet.getString("TABLE_NAME")).then(j -> i.getArgument(2));
                return tableResultSet;
            });
            when(tableResultSet.next()).thenAnswer(i -> nextTable.getAndSet(false));
            when(metaData.getColumns(any(), any(), any(), any())).thenReturn(columnResultSet);
            when(columnResultSet.getString("COLUMN_NAME")).thenReturn("MOCK_COLUMN");
            when(columnResultSet.getInt("DATA_TYPE")).thenReturn(Types.VARCHAR);
            when(columnResultSet.next()).thenAnswer(i -> nextColumn.getAndSet(false));
            when(connection.prepareStatement(any())).thenReturn(preparedStatement);
            when(preparedStatement.executeUpdate()).thenReturn(1);
        } catch (Exception e) {
            throw new ExceptionInInitializerError(e);
        }
    }

}
