/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.ddv.converter;

import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.EffectuatedVaccination;
import dk.vaccinationsregister.schemas._2013._12._01.EffectuationOverviewType;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.convert.converter.Converter;
import org.springframework.lang.NonNull;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;

import java.util.stream.Stream;

import static java.util.stream.Stream.generate;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.params.provider.Arguments.arguments;

@SpringJUnitConfig(DDVMapperConfiguration.class)
class EffectuatedVaccinationConverterTest {

  @NonNull
  private final Converter<EffectuationOverviewType, EffectuatedVaccination> converter;

  @Autowired
  EffectuatedVaccinationConverterTest(
    @NonNull final Converter<EffectuationOverviewType, EffectuatedVaccination> converter) {
    this.converter = converter;
  }

  @SuppressWarnings("unused") // Idea does not pickup @ParameterizedTest using friendlyName
  @DisplayName("Conversion of EffectuatedItemType to EffectuatedVaccination")
  // TODO @ParameterizedTest(name = "{index}: {1}")
  @MethodSource("items")
  void convertNonNullDateTime(@NonNull final EffectuationOverviewType item, @NonNull final String friendly) {
    final EffectuatedVaccination vaccination = converter.convert(item);
    assertNotNull(vaccination);
    //assertEquals(item.getEffectuatedVaccineIdentifier(), vaccination.vaccineIdentifier());
    //assertEquals(item.getEffectuatedVaccinationIdentifier(), vaccination.vaccinationIdentifier());
    //assertEquals(item.getEffectuatedVaccinationVersionIdentifier(), vaccination.vaccinationVersionIdentifier());
    //assertEquals(item.getEffectuatedVaccineName(), vaccination.vaccineName());
    //assertEquals(item.getEffectuatedVaccinationDateTime().toGregorianCalendar().toZonedDateTime().toLocalDateTime(), vaccination.vaccinationDateTime());
  }

  @Test
  void convertNullDateTime() {
    final EffectuationOverviewType item = Tests.newEffectuatedItem();
    item.setEffectuatedDateTime(null);
    assertNull(converter.convert(item));
  }

  @NonNull
  private static Stream<Arguments> items() {
    return generate(Tests::newEffectuatedItem)
      .limit(100)
      .map((item) -> arguments(item, friendly(item)));
  }

  @NonNull
  private static String friendly(@NonNull final EffectuationOverviewType item) {
      return null; // TODO
    //return String.format(ITEM_FORMAT_STRING,
      //item.getEffectuatedVaccineIdentifier(),
      //item.getEffectuatedVaccinationIdentifier(),
      //item.getEffectuatedVaccinationVersionIdentifier(),
      //item.getEffectuatedVaccineName(),
      //item.getEffectuatedVaccinationDateTime().toGregorianCalendar());
  }
}
