/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.lab;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonGetter;
import com.fasterxml.jackson.annotation.JsonProperty;
import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResult;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.time.LocalDateTime;
import java.util.Objects;

public class LabResultImpl implements LabResult {

    private static final String ID_PROPERTY = "id";
    private static final String POSITIVE_PROPERTY = "positive";
    private static final String TYPE_PROPERTY = "type";
    private static final String NAME_PROPERTY = "name";
    private static final String MANUFACTURER_PROPERTY = "manufacturer";
    private static final String CENTER_PROPERTY = "center";
    private static final String SAMPLING_DATE_TIME_PROPERTY = "samplingDateTime";
    private static final String RESULT_DATE_TIME_PROPERTY = "resultDateTime";

    private final long id;
    private final boolean positive;
    private final String type;
    private final String name;
    private final String manufacturer;
    private final String center;
    private final LocalDateTime samplingDateTime;
    private final LocalDateTime resultDateTime;

    @JsonCreator
    LabResultImpl(
        @JsonProperty(ID_PROPERTY) final long id,
        @JsonProperty(POSITIVE_PROPERTY) final boolean positive,
        @JsonProperty(TYPE_PROPERTY) @NonNull final String type,
        @JsonProperty(NAME_PROPERTY) @Nullable final String name,
        @JsonProperty(MANUFACTURER_PROPERTY) @Nullable final String manufacturer,
        @JsonProperty(CENTER_PROPERTY) @Nullable final String center,
        @JsonProperty(SAMPLING_DATE_TIME_PROPERTY) @NonNull final LocalDateTime samplingDateTime,
        @JsonProperty(RESULT_DATE_TIME_PROPERTY) @NonNull final LocalDateTime resultDateTime) {
        this.id = id;
        this.positive = positive;
        this.type = type;
        this.name = name;
        this.manufacturer = manufacturer;
        this.center = center;
        this.samplingDateTime = samplingDateTime;
        this.resultDateTime = resultDateTime;
    }

    @Override
    @JsonGetter
    @JsonProperty(ID_PROPERTY)
    public long id() {
        return id;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof LabResultImpl)) return false;
        final LabResultImpl labResult = (LabResultImpl) o;
        return id == labResult.id;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }

    @Override
    @JsonGetter
    @JsonProperty(POSITIVE_PROPERTY)
    public boolean positive() {
        return positive;
    }

    @NonNull
    @Override
    @JsonGetter
    @JsonProperty(TYPE_PROPERTY)
    public String type() {
        return type;
    }

    @Nullable
    @Override
    @JsonGetter
    @JsonProperty(NAME_PROPERTY)
    public String name() {
        return name;
    }

    @Nullable
    @Override
    @JsonGetter
    @JsonProperty(MANUFACTURER_PROPERTY)
    public String manufacturer() {
        return manufacturer;
    }

    @Nullable
    @Override
    @JsonGetter
    @JsonProperty(CENTER_PROPERTY)
    public String center() {
        return center;
    }

    @NonNull
    @Override
    @JsonGetter
    @JsonProperty(SAMPLING_DATE_TIME_PROPERTY)
    public LocalDateTime samplingDateTime() {
        return samplingDateTime;
    }

    @NonNull
    @Override
    @JsonGetter
    @JsonProperty(RESULT_DATE_TIME_PROPERTY)
    public LocalDateTime resultDateTime() {
        return resultDateTime;
    }
}
