/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.lab;

import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.time.LocalDateTime;

public final class LabResultImplBuilder {

    private long id;
    private boolean positive;
    private String type;
    private String name;
    private String manufacturer;
    private String center;
    private LocalDateTime samplingDateTime;
    private LocalDateTime resultDateTime;

    private LabResultImplBuilder(
        final long id,
        @NonNull final String type,
        @NonNull final LocalDateTime samplingDateTime,
        @NonNull final LocalDateTime resultDateTime) {
        this.id = id;
        this.type = type;
        this.samplingDateTime = samplingDateTime;
        this.resultDateTime = resultDateTime;
    }

    @NonNull
    public static LabResultImplBuilder labResult(
        final long id,
        @NonNull final String type,
        @NonNull final LocalDateTime samplingDateTime,
        @NonNull final LocalDateTime resultDateTime) {
        return new LabResultImplBuilder(id, type, samplingDateTime, resultDateTime);
    }

    @NonNull
    public LabResultImplBuilder id(final long id) {
        this.id = id;
        return this;
    }

    @NonNull
    public LabResultImplBuilder positive(final boolean positive) {
        this.positive = positive;
        return this;
    }

    @NonNull
    public LabResultImplBuilder type(@NonNull final String type) {
        this.type = type;
        return this;
    }

    @NonNull
    public LabResultImplBuilder name(@Nullable final String name) {
        this.name = name;
        return this;
    }

    @NonNull
    public LabResultImplBuilder manufacturer(@Nullable final String manufacturer) {
        this.manufacturer = manufacturer;
        return this;
    }

    @NonNull
    public LabResultImplBuilder center(@Nullable final String center) {
        this.center = center;
        return this;
    }

    @NonNull
    public LabResultImplBuilder samplingDateTime(@Nullable final LocalDateTime samplingDateTime) {
        this.samplingDateTime = samplingDateTime;
        return this;
    }

    @NonNull
    public LabResultImplBuilder resultDateTime(@Nullable final LocalDateTime resultDateTime) {
        this.resultDateTime = resultDateTime;
        return this;
    }

    @NonNull public LabResultImpl build() {
        return new LabResultImpl(id, positive, type, name, manufacturer, center, samplingDateTime, resultDateTime);
    }
}
