/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.vaccination;

import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.CompletedVaccinationPlan;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.EffectuatedVaccination;
import org.springframework.lang.NonNull;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

import static java.util.Comparator.comparing;

public final class CompletedVaccinationPlanBuilder {
    @NonNull private final static LocalDateTime WELL_IN_THE_PAST = LocalDateTime.MIN;
    @NonNull private final SortedSet<EffectuatedVaccination> sorted;
    @NonNull private final List<EffectuatedVaccination> vaccinations;

    private CompletedVaccinationPlanBuilder() {
        sorted = new TreeSet<>(comparing(EffectuatedVaccination::vaccinationDateTime).reversed());
        vaccinations = new ArrayList<>();
    }

    public static CompletedVaccinationPlanBuilder vaccinationPlan() {
        return new CompletedVaccinationPlanBuilder();
    }

    public CompletedVaccinationPlanBuilder add(EffectuatedVaccination vaccination) {
        sorted.add(vaccination);
        vaccinations.add(vaccination);
        return this;
    }

    public CompletedVaccinationPlan build() {
        final LocalDateTime completedBy = sorted.isEmpty() ? WELL_IN_THE_PAST : sorted.first().vaccinationDateTime();
        return new CompletedVaccinationPlanImpl(completedBy, vaccinations);
    }
}
