/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.vaccination;

import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.EffectuatedVaccination;
import org.springframework.lang.NonNull;

import java.time.LocalDateTime;

public final class EffectuatedVaccinationBuilder {

  private long vaccineIdentifier;
  private long vaccinationIdentifier;
  private long vaccinationVersionIdentifier;
  @NonNull private final String vaccineName;
  @NonNull private final LocalDateTime vaccinationDateTime;

  private EffectuatedVaccinationBuilder(
    @NonNull final String vaccineName,
    @NonNull final LocalDateTime vaccinationDateTime) {
    this.vaccineName = vaccineName;
    this.vaccinationDateTime = vaccinationDateTime;
  }

  @NonNull
  public static EffectuatedVaccinationBuilder effectuatedVaccination(
    @NonNull final String vaccineName,
    @NonNull final LocalDateTime vaccinationDateTime) {
    return new EffectuatedVaccinationBuilder(vaccineName, vaccinationDateTime);
  }

  @NonNull
  public EffectuatedVaccinationBuilder vaccineIdentifier(final long vaccineIdentifier) {
    this.vaccineIdentifier = vaccineIdentifier;
    return this;
  }

  @NonNull
  public EffectuatedVaccinationBuilder vaccinationIdentifier(final long vaccinationIdentifier) {
    this.vaccinationIdentifier = vaccinationIdentifier;
    return this;
  }

  @NonNull
  public EffectuatedVaccinationBuilder vaccinationVersionIdentifier(final long vaccinationVersionIdentifier) {
    this.vaccinationVersionIdentifier = vaccinationVersionIdentifier;
    return this;
  }

  @NonNull
  public EffectuatedVaccination build() {
    return new EffectuatedVaccinationImpl(
      vaccineIdentifier,
      vaccinationIdentifier,
      vaccinationVersionIdentifier,
      vaccineName,
      vaccinationDateTime);

  }
}
