/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.vaccination;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonGetter;
import com.fasterxml.jackson.annotation.JsonProperty;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.EffectuatedVaccination;
import org.springframework.lang.NonNull;

import java.time.LocalDateTime;

class EffectuatedVaccinationImpl implements EffectuatedVaccination {

    private static final String VACCINE_IDENTIFIER_PROPERTY = "vaccineIdentifier";
    private static final String VACCINATION_IDENTIFIER_PROPERTY = "vaccinationIdentifier";
    private static final String VACCINATION_VERSION_IDENTIFIER_PROPERTY = "vaccinationVersionIdentifier";
    public static final String VACCINE_NAME_PROPERTY = "vaccineName";
    public static final String VACCINATION_DATE_TIME_PROPERTY = "vaccinationDateTime";

    private final long vaccineIdentifier;
    private final long vaccinationIdentifier;
    private final long vaccinationVersionIdentifier;
    @NonNull private final String vaccineName;
    @NonNull private final LocalDateTime vaccinationDateTime;

    @JsonCreator
    EffectuatedVaccinationImpl(
        @JsonProperty(VACCINE_IDENTIFIER_PROPERTY) final long vaccineIdentifier,
        @JsonProperty(VACCINATION_IDENTIFIER_PROPERTY) final long vaccinationIdentifier,
        @JsonProperty(VACCINATION_VERSION_IDENTIFIER_PROPERTY) final long vaccinationVersionIdentifier,
        @NonNull @JsonProperty(VACCINE_NAME_PROPERTY) final String vaccineName,
        @NonNull @JsonProperty(VACCINATION_DATE_TIME_PROPERTY) final LocalDateTime vaccinationDateTime) {
        this.vaccineIdentifier = vaccineIdentifier;
        this.vaccinationIdentifier = vaccinationIdentifier;
        this.vaccinationVersionIdentifier = vaccinationVersionIdentifier;
        this.vaccineName = vaccineName;
        this.vaccinationDateTime = vaccinationDateTime;
    }

    @JsonProperty(VACCINE_IDENTIFIER_PROPERTY)
    @JsonGetter
    @Override
    public long vaccineIdentifier() {
        return vaccineIdentifier;
    }

    @JsonProperty(VACCINATION_IDENTIFIER_PROPERTY)
    @JsonGetter
    @Override
    public long vaccinationIdentifier() {
        return vaccinationIdentifier;
    }

    @JsonProperty(VACCINATION_VERSION_IDENTIFIER_PROPERTY)
    @JsonGetter
    @Override
    public long vaccinationVersionIdentifier() {
        return vaccinationVersionIdentifier;
    }

    @JsonProperty(VACCINE_NAME_PROPERTY)
    @JsonGetter
    @Override
    @NonNull
    public String vaccineName() {
        return vaccineName;
    }

    @JsonProperty(VACCINATION_DATE_TIME_PROPERTY)
    @JsonGetter
    @Override
    @NonNull
    public LocalDateTime vaccinationDateTime() {
        return vaccinationDateTime;
    }
}
