/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.vaccination;

import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.CompletedVaccinationPlan;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.EffectuatedVaccination;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.Vaccination;
import org.springframework.lang.NonNull;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

public final class VaccinationBuilder {

    @NonNull private final String cpr;
    @NonNull private final List<EffectuatedVaccination> vaccinations;
    @NonNull private final List<CompletedVaccinationPlan> plans;

    private VaccinationBuilder(@NonNull final String cpr) {
        this.cpr = cpr;
        this.vaccinations = new ArrayList<>();
        this.plans = new ArrayList<>();
    }

    @NonNull
    public static VaccinationBuilder cpr(@NonNull final String cpr) {
        return new VaccinationBuilder(cpr);
    }

    @NonNull
    public VaccinationBuilder add(@NonNull final EffectuatedVaccination effectuatedVaccination) {
        vaccinations.add(effectuatedVaccination);
        return this;
    }

    @NonNull
    public VaccinationBuilder add(@NonNull final CompletedVaccinationPlan plan) {
        plans.add(plan);
        return this;
    }

    @NonNull
    public Vaccination build() {
        return new VaccinationImpl(cpr, vaccinations, plans);
    }
}
