/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.vaccination;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonGetter;
import com.fasterxml.jackson.annotation.JsonProperty;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.CompletedVaccinationPlan;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.EffectuatedVaccination;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.Vaccination;
import org.springframework.lang.NonNull;

import java.util.List;

import static java.util.Collections.unmodifiableList;

public class VaccinationImpl implements Vaccination {

    private static final String CPR_NAME = "cpr";
    private static final String EFFECTUATED_VACCINATIONS_NAME = "effectuatedVaccinations";
    private static final String COMPLETED_VACCINATION_PLANS = "completedVaccinationPlans";

    @NonNull private final String cpr;
    @NonNull private final List<EffectuatedVaccination> effectuatedVaccinations;
    @NonNull private final List<CompletedVaccinationPlan> plans;

    @JsonCreator
    VaccinationImpl(
        @NonNull @JsonProperty(CPR_NAME) final String cpr,
        @NonNull @JsonProperty(EFFECTUATED_VACCINATIONS_NAME) final List<EffectuatedVaccination> effectuatedVaccinations,
        @NonNull @JsonProperty(COMPLETED_VACCINATION_PLANS) final List<CompletedVaccinationPlan> plans) {
        this.cpr = cpr;
        this.effectuatedVaccinations = unmodifiableList(effectuatedVaccinations);
        this.plans = plans;
    }

    @JsonGetter
    @JsonProperty(CPR_NAME)
    @Override
    public String cpr() {
        return cpr;
    }

    @JsonGetter
    @JsonProperty(EFFECTUATED_VACCINATIONS_NAME)
    @Override
    public List<EffectuatedVaccination> effectuatedVaccinations() {
        return effectuatedVaccinations;
    }

    @JsonGetter
    @JsonProperty(COMPLETED_VACCINATION_PLANS)
    @Override
    public List<CompletedVaccinationPlan> completedPlans() {
        return plans;
    }
}
