/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.kafka;

import dk.sundhedsdatastyrelsen.cbs.dto.kafka.JobType;
import dk.sundhedsdatastyrelsen.cbs.test.CbsTest;
import org.junit.jupiter.api.Test;

import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotEquals;

@CbsTest
class DataFetcherJobImplTest {

    @Test
    void test_equals() {
        final String cpr = "0101012233";
        final String msgId = UUID.randomUUID().toString();
        final String jobId = UUID.randomUUID().toString();
        final JobType jobType = JobType.FETCH_ALL;

        final DataFetcherJobImpl job1 = new DataFetcherJobImpl(msgId, jobType, jobId, cpr);
        final DataFetcherJobImpl job2 = new DataFetcherJobImpl(msgId, jobType, jobId, cpr);

        assertEquals(job1, job2);
        assertEquals(job1.hashCode(), job2.hashCode());
        assertEquals(job1.toString(), job2.toString());
    }

    @Test
    void test_cpr_not_in_toString() {
        final String msgId = UUID.randomUUID().toString();
        final String jobId = UUID.randomUUID().toString();
        final JobType jobType = JobType.FETCH_ALL;

        final DataFetcherJobImpl job1 = new DataFetcherJobImpl(msgId, jobType, jobId, "0101012233");
        final DataFetcherJobImpl job2 = new DataFetcherJobImpl(msgId, jobType, jobId, "0202023344");

        assertNotEquals(job1, job2);
        assertNotEquals(job1.hashCode(), job2.hashCode());
        assertEquals(job1.toString(), job2.toString());
    }
}
