/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.lab;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.module.SimpleModule;
import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResult;
import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResults;
import dk.sundhedsdatastyrelsen.cbs.test.RandomData;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.springframework.lang.NonNull;

import java.util.stream.Collectors;
import java.util.stream.Stream;

import static dk.sundhedsdatastyrelsen.cbs.dto.impl.lab.LabResultImplBuilder.labResult;
import static java.util.stream.Stream.generate;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

class LabResultImplTest {

    @NonNull private static final RandomData RANDOM_DATA = new RandomData("CBS".hashCode());
    @NonNull private final ObjectMapper objectMapper;

    LabResultImplTest() {
        final SimpleModule module = new SimpleModule()
            .addAbstractTypeMapping(LabResults.class, LabResultsImpl.class)
            .addAbstractTypeMapping(LabResult.class, LabResultImpl.class);
        objectMapper = new ObjectMapper().findAndRegisterModules().registerModule(module).disable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS);
    }

    @MethodSource("result")
    @ParameterizedTest
    void serializeDeserialize(@NonNull final LabResult source) throws JsonProcessingException {
        final String serialized = objectMapper.writeValueAsString(source);
        assertNotNull(serialized);

        final LabResult deserialized = objectMapper.readValue(serialized, LabResult.class);
        assertNotNull(deserialized);
        assertEquals(source.id(), deserialized.id());
        assertEquals(source.positive(), deserialized.positive());
    }

    @MethodSource("results")
    @ParameterizedTest
    void serializeDeserialize(@NonNull final LabResults source) throws JsonProcessingException {
        final String serialized = objectMapper.writeValueAsString(source);
        assertNotNull(serialized);

        final LabResults deserialized = objectMapper.readValue(serialized, LabResults.class);
        assertNotNull(deserialized);
        assertEquals(source.size(), deserialized.size());
    }

    @NonNull
    private static Stream<Arguments> results() {
        return generate(() ->
                generate(LabResultImplTest::newLabResult)
                .limit(RANDOM_DATA.nextInt(10)).collect(Collectors.toCollection(LabResultsImpl::new)))
            .limit(100)
            .map(Arguments::arguments);
    }

    @NonNull
    private static Stream<Arguments> result() {
        return generate(LabResultImplTest::newLabResult)
            .limit(100)
            .map(Arguments::arguments);
    }

    @NonNull
    private static LabResult newLabResult() {
        return labResult(
            RANDOM_DATA.nextLong(),
            RANDOM_DATA.string(10),
            RANDOM_DATA.localDateTime(),
            RANDOM_DATA.localDateTime())
            .center(RANDOM_DATA.nextBoolean() ? RANDOM_DATA.string(10) : null)
            .manufacturer(RANDOM_DATA.nextBoolean() ? RANDOM_DATA.string(10) : null)
            .positive(RANDOM_DATA.nextBoolean())
            .build();
    }

}
