/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.vaccination;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.module.SimpleModule;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.CompletedVaccinationPlan;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.EffectuatedVaccination;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.Vaccination;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

class VaccinationImplTest {

    private final ObjectMapper objectMapper;

    VaccinationImplTest() {
        final SimpleModule module = new SimpleModule()
            .addAbstractTypeMapping(EffectuatedVaccination.class, EffectuatedVaccinationImpl.class)
            .addAbstractTypeMapping(CompletedVaccinationPlan.class, CompletedVaccinationPlanImpl.class)
            .addAbstractTypeMapping(Vaccination.class, VaccinationImpl.class);
        objectMapper = new ObjectMapper().findAndRegisterModules().registerModule(module).disable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS);
    }

    @Test
    void serializeDeserialize() throws JsonProcessingException {
        final Vaccination source = VaccinationBuilder.cpr("cpr").build();

        final String serialized = objectMapper.writeValueAsString(source);
        assertNotNull(serialized);

        final Vaccination deserialized = objectMapper.readValue(serialized, Vaccination.class);
        assertNotNull(deserialized);
        assertEquals(source.cpr(), deserialized.cpr());
        assertEquals(source.effectuatedVaccinations().size(), deserialized.effectuatedVaccinations().size());
    }

}
