/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.test;


import dk.sundhedsdatastyrelsen.cbs.dto.impl.kafka.DataFetcherJobBuilder;
import dk.sundhedsdatastyrelsen.cbs.dto.kafka.DataFetcherJob;
import dk.sundhedsdatastyrelsen.cbs.dto.kafka.JobType;

import java.util.HashSet;
import java.util.Random;
import java.util.Set;
import java.util.UUID;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

public class KafkaTestUtils {
    private static final RandomData RANDOM = new RandomData(new Random().nextInt());

    private KafkaTestUtils() {
    }

    public static DataFetcherJob createRandomDataFetcherJob() {
        JobType randomJobType = getRandomJobType();
        return createRandomDataFetcherJob(randomJobType);
    }

    public static DataFetcherJob createRandomDataFetcherJob(JobType jobType) {
        return DataFetcherJobBuilder.aDataFetcherJob()
            .withCpr(RANDOM.cpr())
            .withType(jobType)
            .withMsgId(UUID.randomUUID().toString())
            .withJobId(UUID.randomUUID().toString())
            .build();
    }


    public static JobType getRandomJobType() {
        int pickRandomEnum = RANDOM.nextInt(JobType.values().length);
        return JobType.values()[pickRandomEnum];
    }

    public static Set<DataFetcherJob> createRandomJobs(int jobCount) {
        return IntStream.range(0, jobCount).mapToObj(i -> createRandomDataFetcherJob()).collect(Collectors.toCollection(HashSet::new));
    }


}
