/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.config;

import dk.sds.nsp.kafka.clients.producer.NSPKafkaProducer;
import dk.sundhedsdatastyrelsen.cbs.dto.kafka.DataFetcherJob;
import dk.sundhedsdatastyrelsen.cbs.kafka.CbsKafkaProducer;
import dk.sundhedsdatastyrelsen.cbs.kafka.CbsKafkaUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.ConfigurableEnvironment;

import java.util.Properties;
import java.util.function.Function;

@Configuration
public class ServiceKafkaConfig {
    private final ConfigurableEnvironment env;

    @Autowired
    public ServiceKafkaConfig(ConfigurableEnvironment env) {
        this.env = env;
    }

    @Bean
    public CbsKafkaProducer cbsKafkaProducer(Function<Properties, NSPKafkaProducer<String, DataFetcherJob>> nspKafkaProducerConstructor) {
        final Properties kafkaProducerProperties = CbsKafkaUtils.getKafkaProperties("kafka.producer.", env);

        final String topic = env.getProperty("kafka.topic");
        if (StringUtils.isEmpty(topic))
            throw new IllegalArgumentException("kafka.topic configuration property missing");

        return new CbsKafkaProducer(topic, CbsKafkaUtils.getProducerInstanceProperties(kafkaProducerProperties), nspKafkaProducerConstructor.apply(kafkaProducerProperties));
    }
}
