-- liquibase formatted sql
-- changeset mbl@nine.dk:create-kids_test_result-table

create or replace table test_result
(
    test_identifier            bigint primary key not null comment 'Unik identifier af prøvesvaret',
    person_identifier          char(10)           not null comment 'CPR-nummer',
    type_of_test               varchar(15)        not null comment 'Description of the type of test that was conducted, e.g. RT-PCR or rapid antigen test.
In the case of a rapid antigen tests, the form should provide details on the manufacturer and commercial name of the test used',
    test_name                  varchar(100)       not null comment 'Commercial or brand name of the RT-PCR or rapid antigen test. Pt. kun for AG-test',
    test_manufacturer          varchar(100)       not null comment 'Manufacturer of the RT-PCR or rapid antigen test. Kun for AG-test',
    timestamp_sample           timestamp          not null comment 'Date when the sample for the test was collected',
    timestamp_result           timestamp          not null comment 'Date when the test was determined',
    result_of_the_test         varchar(15)        not null comment 'Negative (false) or positive (true)',
    testing_centre_or_facility varchar(100)       not null comment 'Name/code of testing centre, facility or a health authority responsible for the testing event. Kun for AG-test',
    timestamp_last_update      timestamp          not null comment 'Date when the test was last updated (in KIDS)',
    sletteflag                 tinyint            not null comment 'Markering af logisk sletning, digi2: 0 for ikke-slettet og 1 for slettet'
);

create or replace index ix__test_result_cpr on test_result (person_identifier);
create or replace index ix__test_result_delete on test_result (timestamp_result, result_of_the_test);

create or replace view test_result_view as
select
    test_identifier id,
    person_identifier cpr,
    type_of_test type,
    test_name name,
    test_manufacturer manufacturer,
    timestamp_sample sample_timestamp,
    timestamp_result result_timestamp,
    result_of_the_test result,
    testing_centre_or_facility center,
    timestamp_last_update creation_timestamp
from test_result
where not sletteflag;


