-- liquibase formatted sql
-- changeset cdj@trifork.com:initial-database-setup

create or replace table vaccination
(
    cpr               char(10)                               not null comment 'the unique id (cpr number) of the subject to whom a particular record belongs',
    trace_id          varchar(50)                            not null comment 'trace id for this record',
    creation_time     timestamp(6) default current_timestamp not null comment 'time the data for this record was created. nb: client-generated',
    invalidation_time timestamp                              null comment 'time the invalidation record was received',
    data              longtext                               null comment 'the actual data encoded using JSON',
    source            longtext                               null comment 'the raw data encoded using JSON',
    constraint pk__vaccination primary key (cpr, creation_time) comment 'primary key constraint',
    constraint chk__vaccination check (not (data is null and invalidation_time is null))
)
    engine = innodb
    comment 'table storing all records of vaccinations including history'
    -- We want to be able to wipe an entire month worth of data in a single fast operation, cf. https://mariadb.com/kb/en/big-deletes/
    partition by range (floor(unix_timestamp(creation_time))) (
        partition 2021_01 values less than (floor(unix_timestamp('2021-02-01 00:00:00'))),
        partition 2021_02 values less than (floor(unix_timestamp('2021-03-01 00:00:00'))),
        partition 2021_03 values less than (floor(unix_timestamp('2021-04-01 00:00:00'))),
        partition 2021_04 values less than (floor(unix_timestamp('2021-05-01 00:00:00'))),
        partition 2021_05 values less than (floor(unix_timestamp('2021-06-01 00:00:00'))),
        partition 2021_06 values less than (floor(unix_timestamp('2021-07-01 00:00:00'))),
        partition 2021_07 values less than (floor(unix_timestamp('2021-08-01 00:00:00'))),
        partition 2021_08 values less than (floor(unix_timestamp('2021-09-01 00:00:00'))),
        partition 2021_09 values less than (floor(unix_timestamp('2021-10-01 00:00:00'))),
        partition 2021_10 values less than (floor(unix_timestamp('2021-11-01 00:00:00'))),
        partition 2021_11 values less than (floor(unix_timestamp('2021-12-01 00:00:00'))),
        partition 2021_12 values less than (floor(unix_timestamp('2022-01-01 00:00:00'))),

        partition 2022_01 values less than (floor(unix_timestamp('2022-02-01 00:00:00'))),
        partition 2022_02 values less than (floor(unix_timestamp('2022-03-01 00:00:00'))),
        partition 2022_03 values less than (floor(unix_timestamp('2022-04-01 00:00:00'))),
        partition 2022_04 values less than (floor(unix_timestamp('2022-05-01 00:00:00'))),
        partition 2022_05 values less than (floor(unix_timestamp('2022-06-01 00:00:00'))),
        partition 2022_06 values less than (floor(unix_timestamp('2022-07-01 00:00:00'))),
        partition 2022_07 values less than (floor(unix_timestamp('2022-08-01 00:00:00'))),
        partition 2022_08 values less than (floor(unix_timestamp('2022-09-01 00:00:00'))),
        partition 2022_09 values less than (floor(unix_timestamp('2022-10-01 00:00:00'))),
        partition 2022_10 values less than (floor(unix_timestamp('2022-11-01 00:00:00'))),
        partition 2022_11 values less than (floor(unix_timestamp('2022-12-01 00:00:00'))),
        partition 2022_12 values less than (floor(unix_timestamp('2023-01-01 00:00:00'))),

        partition 2023_01 values less than (floor(unix_timestamp('2023-02-01 00:00:00'))),
        partition 2023_02 values less than (floor(unix_timestamp('2023-03-01 00:00:00'))),
        partition 2023_03 values less than (floor(unix_timestamp('2023-04-01 00:00:00'))),
        partition 2023_04 values less than (floor(unix_timestamp('2023-05-01 00:00:00'))),
        partition 2023_05 values less than (floor(unix_timestamp('2023-06-01 00:00:00'))),
        partition 2023_06 values less than (floor(unix_timestamp('2023-07-01 00:00:00'))),
        partition 2023_07 values less than (floor(unix_timestamp('2023-08-01 00:00:00'))),
        partition 2023_08 values less than (floor(unix_timestamp('2023-09-01 00:00:00'))),
        partition 2023_09 values less than (floor(unix_timestamp('2023-10-01 00:00:00'))),
        partition 2023_10 values less than (floor(unix_timestamp('2023-11-01 00:00:00'))),
        partition 2023_11 values less than (floor(unix_timestamp('2023-12-01 00:00:00'))),
        partition 2023_12 values less than (floor(unix_timestamp('2024-01-01 00:00:00'))),

        partition 2024_01 values less than (floor(unix_timestamp('2024-02-01 00:00:00'))),
        partition 2024_02 values less than (floor(unix_timestamp('2024-03-01 00:00:00'))),
        partition 2024_03 values less than (floor(unix_timestamp('2024-04-01 00:00:00'))),
        partition 2024_04 values less than (floor(unix_timestamp('2024-05-01 00:00:00'))),
        partition 2024_05 values less than (floor(unix_timestamp('2024-06-01 00:00:00'))),
        partition 2024_06 values less than (floor(unix_timestamp('2024-07-01 00:00:00'))),
        partition 2024_07 values less than (floor(unix_timestamp('2024-08-01 00:00:00'))),
        partition 2024_08 values less than (floor(unix_timestamp('2024-09-01 00:00:00'))),
        partition 2024_09 values less than (floor(unix_timestamp('2024-10-01 00:00:00'))),
        partition 2024_10 values less than (floor(unix_timestamp('2024-11-01 00:00:00'))),
        partition 2024_11 values less than (floor(unix_timestamp('2024-12-01 00:00:00'))),
        partition 2024_12 values less than (floor(unix_timestamp('2025-01-01 00:00:00'))),

        partition 2025_01 values less than (floor(unix_timestamp('2025-02-01 00:00:00'))),
        partition 2025_02 values less than (floor(unix_timestamp('2025-03-01 00:00:00'))),
        partition 2025_03 values less than (floor(unix_timestamp('2025-04-01 00:00:00'))),
        partition 2025_04 values less than (floor(unix_timestamp('2025-05-01 00:00:00'))),
        partition 2025_05 values less than (floor(unix_timestamp('2025-06-01 00:00:00'))),
        partition 2025_06 values less than (floor(unix_timestamp('2025-07-01 00:00:00'))),
        partition 2025_07 values less than (floor(unix_timestamp('2025-08-01 00:00:00'))),
        partition 2025_08 values less than (floor(unix_timestamp('2025-09-01 00:00:00'))),
        partition 2025_09 values less than (floor(unix_timestamp('2025-10-01 00:00:00'))),
        partition 2025_10 values less than (floor(unix_timestamp('2025-11-01 00:00:00'))),
        partition 2025_11 values less than (floor(unix_timestamp('2025-12-01 00:00:00'))),
        partition 2025_12 values less than (floor(unix_timestamp('2026-01-01 00:00:00')))
        );

create or replace view vaccination_view as
select v.cpr,
       v.trace_id,
       v.creation_time,
       v.invalidation_time,
       v.data,
       v.source
from (select cpr, max(creation_time) max_creation_time
      from vaccination
      where data is not null
         or invalidation_time <= current_timestamp
      group by cpr) s
         join vaccination v on v.cpr = s.cpr and v.creation_time = s.max_creation_time;

create or replace table lab_result
(
    cpr               char(10)                               not null comment 'the unique id (cpr number) of the subject to whom a particular record belongs',
    trace_id          varchar(50)                            not null comment 'trace id for this record',
    creation_time     timestamp(6) default current_timestamp not null comment 'time the data for this record was created. nb: client-generated',
    invalidation_time timestamp                              null comment 'time the invalidation record was received',
    data              longtext                               null comment 'the actual data encoded using JSON',
    source            longtext                               null comment 'the raw data encoded using JSON',
    constraint pk__lab_result primary key (cpr, creation_time) comment 'primary key constraint',
    constraint chk__lab_result check (not (data is null and invalidation_time is null))
)
    engine = innodb
    comment 'table storing all records of lab_result including history'
    -- We want to be able to wipe an entire month worth of data in a single fast operation, cf. https://mariadb.com/kb/en/big-deletes/
    partition by range (floor(unix_timestamp(creation_time))) (
        partition 2021_01 values less than (floor(unix_timestamp('2021-02-01 00:00:00'))),
        partition 2021_02 values less than (floor(unix_timestamp('2021-03-01 00:00:00'))),
        partition 2021_03 values less than (floor(unix_timestamp('2021-04-01 00:00:00'))),
        partition 2021_04 values less than (floor(unix_timestamp('2021-05-01 00:00:00'))),
        partition 2021_05 values less than (floor(unix_timestamp('2021-06-01 00:00:00'))),
        partition 2021_06 values less than (floor(unix_timestamp('2021-07-01 00:00:00'))),
        partition 2021_07 values less than (floor(unix_timestamp('2021-08-01 00:00:00'))),
        partition 2021_08 values less than (floor(unix_timestamp('2021-09-01 00:00:00'))),
        partition 2021_09 values less than (floor(unix_timestamp('2021-10-01 00:00:00'))),
        partition 2021_10 values less than (floor(unix_timestamp('2021-11-01 00:00:00'))),
        partition 2021_11 values less than (floor(unix_timestamp('2021-12-01 00:00:00'))),
        partition 2021_12 values less than (floor(unix_timestamp('2022-01-01 00:00:00'))),

        partition 2022_01 values less than (floor(unix_timestamp('2022-02-01 00:00:00'))),
        partition 2022_02 values less than (floor(unix_timestamp('2022-03-01 00:00:00'))),
        partition 2022_03 values less than (floor(unix_timestamp('2022-04-01 00:00:00'))),
        partition 2022_04 values less than (floor(unix_timestamp('2022-05-01 00:00:00'))),
        partition 2022_05 values less than (floor(unix_timestamp('2022-06-01 00:00:00'))),
        partition 2022_06 values less than (floor(unix_timestamp('2022-07-01 00:00:00'))),
        partition 2022_07 values less than (floor(unix_timestamp('2022-08-01 00:00:00'))),
        partition 2022_08 values less than (floor(unix_timestamp('2022-09-01 00:00:00'))),
        partition 2022_09 values less than (floor(unix_timestamp('2022-10-01 00:00:00'))),
        partition 2022_10 values less than (floor(unix_timestamp('2022-11-01 00:00:00'))),
        partition 2022_11 values less than (floor(unix_timestamp('2022-12-01 00:00:00'))),
        partition 2022_12 values less than (floor(unix_timestamp('2023-01-01 00:00:00'))),

        partition 2023_01 values less than (floor(unix_timestamp('2023-02-01 00:00:00'))),
        partition 2023_02 values less than (floor(unix_timestamp('2023-03-01 00:00:00'))),
        partition 2023_03 values less than (floor(unix_timestamp('2023-04-01 00:00:00'))),
        partition 2023_04 values less than (floor(unix_timestamp('2023-05-01 00:00:00'))),
        partition 2023_05 values less than (floor(unix_timestamp('2023-06-01 00:00:00'))),
        partition 2023_06 values less than (floor(unix_timestamp('2023-07-01 00:00:00'))),
        partition 2023_07 values less than (floor(unix_timestamp('2023-08-01 00:00:00'))),
        partition 2023_08 values less than (floor(unix_timestamp('2023-09-01 00:00:00'))),
        partition 2023_09 values less than (floor(unix_timestamp('2023-10-01 00:00:00'))),
        partition 2023_10 values less than (floor(unix_timestamp('2023-11-01 00:00:00'))),
        partition 2023_11 values less than (floor(unix_timestamp('2023-12-01 00:00:00'))),
        partition 2023_12 values less than (floor(unix_timestamp('2024-01-01 00:00:00'))),

        partition 2024_01 values less than (floor(unix_timestamp('2024-02-01 00:00:00'))),
        partition 2024_02 values less than (floor(unix_timestamp('2024-03-01 00:00:00'))),
        partition 2024_03 values less than (floor(unix_timestamp('2024-04-01 00:00:00'))),
        partition 2024_04 values less than (floor(unix_timestamp('2024-05-01 00:00:00'))),
        partition 2024_05 values less than (floor(unix_timestamp('2024-06-01 00:00:00'))),
        partition 2024_06 values less than (floor(unix_timestamp('2024-07-01 00:00:00'))),
        partition 2024_07 values less than (floor(unix_timestamp('2024-08-01 00:00:00'))),
        partition 2024_08 values less than (floor(unix_timestamp('2024-09-01 00:00:00'))),
        partition 2024_09 values less than (floor(unix_timestamp('2024-10-01 00:00:00'))),
        partition 2024_10 values less than (floor(unix_timestamp('2024-11-01 00:00:00'))),
        partition 2024_11 values less than (floor(unix_timestamp('2024-12-01 00:00:00'))),
        partition 2024_12 values less than (floor(unix_timestamp('2025-01-01 00:00:00'))),

        partition 2025_01 values less than (floor(unix_timestamp('2025-02-01 00:00:00'))),
        partition 2025_02 values less than (floor(unix_timestamp('2025-03-01 00:00:00'))),
        partition 2025_03 values less than (floor(unix_timestamp('2025-04-01 00:00:00'))),
        partition 2025_04 values less than (floor(unix_timestamp('2025-05-01 00:00:00'))),
        partition 2025_05 values less than (floor(unix_timestamp('2025-06-01 00:00:00'))),
        partition 2025_06 values less than (floor(unix_timestamp('2025-07-01 00:00:00'))),
        partition 2025_07 values less than (floor(unix_timestamp('2025-08-01 00:00:00'))),
        partition 2025_08 values less than (floor(unix_timestamp('2025-09-01 00:00:00'))),
        partition 2025_09 values less than (floor(unix_timestamp('2025-10-01 00:00:00'))),
        partition 2025_10 values less than (floor(unix_timestamp('2025-11-01 00:00:00'))),
        partition 2025_11 values less than (floor(unix_timestamp('2025-12-01 00:00:00'))),
        partition 2025_12 values less than (floor(unix_timestamp('2026-01-01 00:00:00')))
        );

create or replace view lab_result_view as
select l.cpr,
       l.trace_id,
       l.creation_time,
       l.invalidation_time,
       l.data,
       l.source
from (select cpr, max(creation_time) max_creation_time
      from lab_result
      where data is not null
         or invalidation_time <= current_timestamp
      group by cpr) s
         join lab_result l on l.cpr = s.cpr and l.creation_time = s.max_creation_time;
