/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package se.digg.dgc.signatures;

import java.security.cert.X509Certificate;
import java.util.List;

/**
 * A functional interface for finding certificates that may be used to verify the signature of a DGC (see
 * {@link DefaultHcertVerifier}).
 * 
 * @author Martin Lindström (martin@idsec.se)
 * @author Henrik Bengtsson (extern.henrik.bengtsson@digg.se)
 * @author Henric Norlander (extern.henric.norlander@digg.se)
 */
@FunctionalInterface
public interface CertificateProvider {

  /**
   * Given a country code and a key identifier the method finds all certificates that matches this criteria.
   * <p>
   * At least one of the criteria is set in a call.
   * </p>
   * <p>
   * If the key identifier (kid) is {@code null} the provider should return all certificates for the given country.
   * </p>
   * <p>
   * If the country code is {@code null} the provider should return all certificates matching the key identifier.
   * </p>
   * 
   * @param country
   *          the two-letter country code
   * @param kid
   *          the key identifier
   * @return a list of certificates (never null)
   */
  List<X509Certificate> getCertificates(final String country, final byte[] kid);

}
