/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package se.digg.dgc.service.interop;

import com.upokecenter.cbor.CBORObject;
import org.apache.commons.io.FileUtils;
import org.junit.Assert;
import org.junit.Ignore;
import org.junit.Test;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import se.digg.dgc.encoding.Base45;
import se.digg.dgc.encoding.Zlib;
import se.digg.dgc.encoding.impl.DefaultBarcodeDecoder;
import se.digg.dgc.signatures.DGCSignatureVerifier;
import se.digg.dgc.signatures.impl.DefaultDGCSignatureVerifier;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.security.cert.X509Certificate;
import java.util.Arrays;

/**
 * Interoperability tests with other implementations
 *
 * @author Martin Lindström (martin@idsec.se)
 * @author Henrik Bengtsson (extern.henrik.bengtsson@digg.se)
 * @author Henric Norlander (extern.henric.norlander@digg.se)
 */
public class InteropConsumeTest {

  @Test
  @Ignore
  public void testDecodeFromBarcode() throws Exception {
    final byte[] image = getFile(new ClassPathResource("interop/hcert-testdata/test1.png"));
    final X509Certificate cert = null; //getCertificate(new ClassPathResource("interop/hcert-testdata/issuer1.crt"));

    // Decode barcode into the Base45 string ...
    final DefaultBarcodeDecoder decoder = new DefaultBarcodeDecoder();
    final String base45 = decoder.decodeToString(image, StandardCharsets.US_ASCII);

    // Assert the the prefix is there.
    Assert.assertTrue(base45.startsWith("HC1"));

    // Base45 decode
    final byte[] compressedCwt = Base45.getDecoder().decode(base45.substring(3));

    // Decompress
    final byte[] cwt = Zlib.decompress(compressedCwt, true);

    // Verify signature
    final DefaultDGCSignatureVerifier verifier = new DefaultDGCSignatureVerifier();
    DGCSignatureVerifier.Result result = verifier.verify(cwt, (c,k) -> Arrays.asList(cert));

    System.out.println(String.format("CWT: issuing-country='%s', issued-at='%s', expires='%s',signer-cert='%s'",
      result.getCountry(), result.getIssuedAt(), result.getExpires(), result.getSignerCertificate().getSubjectX500Principal()));

    // Dump HCERT contents in JSON
    final CBORObject dgcObject = CBORObject.DecodeFromBytes(result.getDgcPayload());
    System.out.println(dgcObject.ToJSONString());

    // Deserialize into our Java representation ...
//    final ObjectMapper cborMapper = new CBORMapper();
//    final DigitalGreenCertificate dgc = cborMapper.readValue(result.getHcert(), DigitalGreenCertificate.class);
  }

  private static byte[] getFile(final Resource resource) throws IOException {
    return FileUtils.readFileToByteArray(resource.getFile());
  }


}
