/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs;

import org.apache.http.HttpResponse;
import org.apache.log4j.Logger;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotEquals;

class ServletITCase extends AbstractITCase {
    private static final Logger log = Logger.getLogger(ServletITCase.class);

    @Test
    void testIsAliveServletOKStatus() throws Exception {
        String uri = super.getItAddress() + "/" + CONTEXT_ROOT + "/isalive";
        log.info("Testing isalive at " + uri);
        HttpResponse response = httpGet(uri);
        log.info("isalive servlet response: " + response);

        int code = response.getStatusLine().getStatusCode();
        assertEquals(200, code);
    }

    @Test
    void testNonExistingServletNotFoundStatus() throws Exception {
        log.info("Testing nonexisting service");
        HttpResponse response = httpGet(super.getItAddress() + "/cbs/nonExistingService");
        log.info("Response from nonexisting service: " + response);

        int code = response.getStatusLine().getStatusCode();
        assertNotEquals(200, code);
    }
}
