/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.configuration;

import dk.sosi.seal.OIOIDWSFactory;
import dk.sosi.seal.vault.ClasspathCredentialVault;
import dk.sosi.seal.vault.CredentialVault;
import dk.sundhedsdatastyrelsen.cbs.webservice.idws.IdwsTokenHelper;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.CoronaPassPort;
import dk.sundhedsdatastyrelsen.coronapass._2021._03.coronapass_web_service.CoronaPassPortService;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.ImportResource;
import org.springframework.context.annotation.PropertySource;

import javax.xml.ws.BindingProvider;
import java.util.Properties;

@Configuration
@ImportResource({ "classpath:META-INF/cxf/cxf.xml"})
@PropertySource("classpath:/application.properties")
public class IntegrationTestConfiguration {
	private static final Logger log = Logger.getLogger(IntegrationTestConfiguration.class);

	@Value("${service.host}")
	String serviceHost;

	@Bean
	public CoronaPassPort coronaPassService() {
		CoronaPassPortService service = new CoronaPassPortService();
		CoronaPassPort port = service.getCoronaPassPortSoap11();

		BindingProvider bp = (BindingProvider) port;
		String webServiceAddress = System.getProperty("WebServiceAddress", serviceHost);
		log.info("Requesting WebServiceAddress: " + webServiceAddress);
		bp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, webServiceAddress);

		Client client = ClientProxy.getClient(port);
		client.getInInterceptors().add(new LoggingInInterceptor());
		client.getOutInterceptors().add(new LoggingOutInterceptor());
		client.getOutInterceptors().add(idwsRequestDecorator());
		client.getInInterceptors().add(libertyResponseValidator());

		return port;
	}

	@Bean
	public CredentialVault trustVault() {
		return new ClasspathCredentialVault(new Properties(), "trust.jks", "Test1234");
	}
	
	@Bean
	public XmlDocumentResponseInterceptor libertyResponseValidator() {
		XmlDocumentResponseInterceptor validator = new XmlDocumentResponseInterceptor();
		return validator;
	}
	
	@Bean
	public LibertyRequestDecoratorInterceptor idwsRequestDecorator() {
		return new LibertyRequestDecoratorInterceptor();
	}
	
	@Bean
	public OIOIDWSFactory oioidwsFactory() {
		IdwsTokenHelper ith = new IdwsTokenHelper();
		return ith.getOioIdwsFactory();
	}
}
