/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.kids.dao;

import org.springframework.lang.NonNull;

import java.time.LocalDateTime;

/**
 * <em>Immutable</em> Data Access Object for a covid 19 lab result in the KIDS mirror database
 */
public interface KidsLabResult {

    String CATALOG_NAME = null;
    String SCHEMA_NAME = null;
    String SELECT_TABLE_NAME = "test_result_view";
    String INSERT_TABLE_NAME = "test_results";

    /**
     * Gets the testing center
     * @return the testing center
     */
    @NonNull String center();
    /**
     * Gets the cpr number of the tested person
     * @return the cpr number
     */
    @NonNull String cpr();
    /**
     * Gets the datetime the record was created or updated
     * @return the datetime
     */
    @NonNull LocalDateTime creationTime();
    /**
     * Gets the identifier of the test
     * @return the identifier
     */
    long id();
    /**
     * Gets the manufacturer of the test
     * @return the manufacturer
     */
    @NonNull String manufacturer();
    /**
     * Gets the name of the test
     * @return the name
     */
    @NonNull String name();
    /**
     * Gets the result of the test
     * @return the result
     */
    @NonNull String result();
    /**
     * Gets the datetime the result was made
     * @return the datetime
     */
    @NonNull LocalDateTime resultTime();
    /**
     * Gets the datetime the sample was taken
     * @return the datetime
     */
    @NonNull LocalDateTime sampleTime();
    /**
     * Gets the type of the test
     * @return the type
     */
    @NonNull String type();
}
