/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.kids.dao;

import dk.sundhedsdatastyrelsen.fluent.spring.jdbc.Mappers;
import dk.sundhedsdatastyrelsen.fluent.spring.jdbc.SearchMapper;

import java.util.function.Function;

import static dk.sundhedsdatastyrelsen.fluent.spring.jdbc.TableMetaDataBuilder.tableMetaData;
import static dk.sundhedsdatastyrelsen.kids.dao.KidsLabResult.CATALOG_NAME;
import static dk.sundhedsdatastyrelsen.kids.dao.KidsLabResult.INSERT_TABLE_NAME;
import static dk.sundhedsdatastyrelsen.kids.dao.KidsLabResult.SCHEMA_NAME;
import static dk.sundhedsdatastyrelsen.kids.dao.KidsLabResult.SELECT_TABLE_NAME;
import static dk.sundhedsdatastyrelsen.kids.dao.KidsLabResultImpl.CENTER_COLUMN_LABEL;
import static dk.sundhedsdatastyrelsen.kids.dao.KidsLabResultImpl.CPR_COLUMN_LABEL;
import static dk.sundhedsdatastyrelsen.kids.dao.KidsLabResultImpl.CREATION_TIME_COLUMN_LABEL;
import static dk.sundhedsdatastyrelsen.kids.dao.KidsLabResultImpl.ID_COLUMN_LABEL;
import static dk.sundhedsdatastyrelsen.kids.dao.KidsLabResultImpl.MANUFACTURER_COLUMN_LABEL;
import static dk.sundhedsdatastyrelsen.kids.dao.KidsLabResultImpl.NAME_COLUMN_LABEL;
import static dk.sundhedsdatastyrelsen.kids.dao.KidsLabResultImpl.RESULT_COLUMN_LABEL;
import static dk.sundhedsdatastyrelsen.kids.dao.KidsLabResultImpl.RESULT_TIME_COLUMN_LABEL;
import static dk.sundhedsdatastyrelsen.kids.dao.KidsLabResultImpl.SAMPLE_TIME_COLUMN_LABEL;
import static dk.sundhedsdatastyrelsen.kids.dao.KidsLabResultImpl.TYPE_COLUMN_LABEL;

public final class KidsMappers {

    private KidsMappers() { }

    public static SearchMapper<KidsLabResultImpl, String> mapper() {
        return Mappers.mapper(
            String.class,
            tableMetaData(INSERT_TABLE_NAME).schemaName(SCHEMA_NAME).catalogName(CATALOG_NAME).build(),
            tableMetaData(SELECT_TABLE_NAME).schemaName(SCHEMA_NAME).catalogName(CATALOG_NAME).build(),
            KidsLabResultImpl::new)
            .addStringMapping(CENTER_COLUMN_LABEL, KidsLabResultImpl::center, KidsLabResultImpl::center)
            .addSearchStringMapping(CPR_COLUMN_LABEL, KidsLabResultImpl::cpr, KidsLabResultImpl::cpr, Function.identity())
            .addTimestampToLocalDateTimeMapping(CREATION_TIME_COLUMN_LABEL, KidsLabResultImpl::creationTime, KidsLabResultImpl::creationTime)
            .addLongMapping(ID_COLUMN_LABEL, KidsLabResultImpl::id, KidsLabResultImpl::id)
            .addStringMapping(MANUFACTURER_COLUMN_LABEL, KidsLabResultImpl::manufacturer, KidsLabResultImpl::manufacturer)
            .addStringMapping(NAME_COLUMN_LABEL, KidsLabResultImpl::name, KidsLabResultImpl::name)
            .addStringMapping(RESULT_COLUMN_LABEL, KidsLabResultImpl::result, KidsLabResultImpl::result)
            .addTimestampToLocalDateTimeMapping(RESULT_TIME_COLUMN_LABEL, KidsLabResultImpl::resultTime, KidsLabResultImpl::resultTime)
            .addTimestampToLocalDateTimeMapping(SAMPLE_TIME_COLUMN_LABEL, KidsLabResultImpl::sampleTime, KidsLabResultImpl::sampleTime)
            .addStringMapping(TYPE_COLUMN_LABEL, KidsLabResultImpl::type, KidsLabResultImpl::type)
            .build();
    }
}
