/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.kids.dao;

import dk.sundhedsdatastyrelsen.cbs.test.RandomData;
import org.apache.commons.lang3.tuple.Triple;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.springframework.lang.NonNull;

import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.stream.Stream;

import static org.junit.jupiter.api.Assertions.assertEquals;

class KidsLabResultImplTest {

    private static final RandomData RANDOM_DATA = new RandomData("CBS".hashCode());
    private static final int MIN_VALUES_PER_PROPERTY = 10;
    private static final int MAX_VALUES_PER_PROPERTY = 30;

    @ParameterizedTest(name = "[{index}] - testing if property \"{0}\" works for value \"{2}\"")
    @MethodSource("properties")
    @DisplayName("Property test")
    <T> void test(
        @NonNull final String propertyName,
        @NonNull final BiFunction<KidsLabResultImpl, T, T> setAndGet,
        @NonNull final T value) {
        final KidsLabResultImpl result = new KidsLabResultImpl();
        assertEquals(value, setAndGet.apply(result, value), String.format("Property %s is not working", propertyName));
    }

    @NonNull
    private static Stream<Arguments> properties() {
        return Stream.of(
                setAndGet("center", KidsLabResultImpl::center, KidsLabResult::center, RANDOM_DATA.strings(20, MIN_VALUES_PER_PROPERTY, MAX_VALUES_PER_PROPERTY)),
                setAndGet("cpr", KidsLabResultImpl::cpr, KidsLabResult::cpr, RANDOM_DATA.strings(10, MIN_VALUES_PER_PROPERTY, MAX_VALUES_PER_PROPERTY)),
                setAndGet("creationTime", KidsLabResultImpl::creationTime, KidsLabResult::creationTime, RANDOM_DATA.localDateTimes(MIN_VALUES_PER_PROPERTY, MAX_VALUES_PER_PROPERTY)),
                setAndGet("id", KidsLabResultImpl::id, KidsLabResult::id, RANDOM_DATA.longs(MIN_VALUES_PER_PROPERTY, MAX_VALUES_PER_PROPERTY)),
                setAndGet("manufacturer", KidsLabResultImpl::manufacturer, KidsLabResult::manufacturer, RANDOM_DATA.strings(20, MIN_VALUES_PER_PROPERTY, MAX_VALUES_PER_PROPERTY)),
                setAndGet("name", KidsLabResultImpl::name, KidsLabResult::name, RANDOM_DATA.strings(20, MIN_VALUES_PER_PROPERTY, MAX_VALUES_PER_PROPERTY)),
                setAndGet("result", KidsLabResultImpl::result, KidsLabResult::result, RANDOM_DATA.strings(15, MIN_VALUES_PER_PROPERTY, MAX_VALUES_PER_PROPERTY)),
                setAndGet("resultTime", KidsLabResultImpl::resultTime, KidsLabResult::resultTime, RANDOM_DATA.localDateTimes(MIN_VALUES_PER_PROPERTY, MAX_VALUES_PER_PROPERTY)),
                setAndGet("sampleTime", KidsLabResultImpl::sampleTime, KidsLabResult::sampleTime, RANDOM_DATA.localDateTimes(MIN_VALUES_PER_PROPERTY, MAX_VALUES_PER_PROPERTY)),
                setAndGet("type", KidsLabResultImpl::type, KidsLabResult::type, RANDOM_DATA.strings(20, MIN_VALUES_PER_PROPERTY, MAX_VALUES_PER_PROPERTY)))
            .flatMap(triple -> triple.getRight().map(value -> Arguments.arguments(triple.getMiddle(), triple.getLeft(), value)));
    }

    private static <T> Triple<BiFunction<KidsLabResultImpl, T, T>, String,  Stream<T>> setAndGet(
        @NonNull final String propertyName,
        @NonNull final BiFunction<KidsLabResultImpl, T, KidsLabResultImpl> setter,
        @NonNull final Function<KidsLabResult, T> getter,
        @NonNull final Stream<T> values) {
        return Triple.of(setter.andThen(getter), propertyName, values);
    }

}
